@@ -1,3 +1,5 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
+
 /*
  *  (GLABELS) Label and Business Card Creation program for GNOME
  *
@@ -23,8 +25,6 @@
 #ifndef __LABEL_BARCODE_H__
 #define __LABEL_BARCODE_H__
 
-#include <gtk/gtkenums.h>
-#include <libgnomeprint/gnome-font.h>
 #include "bc.h"
 #include "text-node.h"
 #include "label-object.h"
@@ -46,7 +46,7 @@ typedef struct _glLabelBarcodePrivate   glLabelBarcodePrivate;
 struct _glLabelBarcode {
 	glLabelObject            object;
 
-	glLabelBarcodePrivate   *private;
+	glLabelBarcodePrivate   *priv;
 };
 
 struct _glLabelBarcodeClass {
