@@ -1,9 +1,11 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
+
 /*
  *  (GLABELS) Label and Business Card Creation program for GNOME
  *
  *  label_object.c:  GLabels label object base class
  *
- *  Copyright (C) 2001-2002  Jim Evins <evins@snaught.com>.
+ *  Copyright (C) 2001-2007  Jim Evins <evins@snaught.com>.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -26,7 +28,7 @@
 #include <glib/gmem.h>
 #include <glib/gstrfuncs.h>
 #include <glib/gmessages.h>
-#include <libart_lgpl/libart.h>
+#include <math.h>
 
 #include "marshal.h"
 
@@ -49,7 +51,7 @@ struct _glLabelObjectPrivate {
 	gchar             *name;
 	gdouble            x, y;
 	gdouble            w, h;
-	gdouble            affine[6];
+        cairo_matrix_t     matrix;
 
 	gdouble            aspect_ratio;
 
@@ -66,6 +68,7 @@ enum {
 	FLIP_ROTATE,
 	TOP,
 	BOTTOM,
+        REMOVED,
 	LAST_SIGNAL
 };
 
@@ -73,8 +76,6 @@ enum {
 /* Private globals.                                       */
 /*========================================================*/
 
-static GObjectClass *parent_class = NULL;
-
 static guint signals[LAST_SIGNAL] = {0};
 
 static guint instance = 0;
@@ -83,8 +84,6 @@ static guint instance = 0;
 /* Private function prototypes.                           */
 /*========================================================*/
 
-static void gl_label_object_class_init    (glLabelObjectClass *klass);
-static void gl_label_object_instance_init (glLabelObject      *object);
 static void gl_label_object_finalize      (GObject            *object);
 
 static void merge_changed_cb              (glLabel            *label,
@@ -94,58 +93,29 @@ static void set_size                      (glLabelObject      *object,
 					   gdouble             w,
 					   gdouble             h);
 
-static void get_size                      (glLabelObject      *object,
-					   gdouble            *w,
-					   gdouble            *h);
 
 
 /*****************************************************************************/
 /* Boilerplate object stuff.                                                 */
 /*****************************************************************************/
-GType
-gl_label_object_get_type (void)
-{
-	static GType type = 0;
-
-	if (!type) {
-		static const GTypeInfo info = {
-			sizeof (glLabelObjectClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) gl_label_object_class_init,
-			NULL,
-			NULL,
-			sizeof (glLabelObject),
-			0,
-			(GInstanceInitFunc) gl_label_object_instance_init,
-			NULL
-		};
-
-		type = g_type_register_static (G_TYPE_OBJECT,
-					       "glLabelObject", &info, 0);
-	}
-
-	return type;
-}
+G_DEFINE_TYPE (glLabelObject, gl_label_object, G_TYPE_OBJECT);
 
 static void
-gl_label_object_class_init (glLabelObjectClass *klass)
+gl_label_object_class_init (glLabelObjectClass *class)
 {
-	GObjectClass       *gobject_class = (GObjectClass *) klass;
-	glLabelObjectClass *object_class  = (glLabelObjectClass *) klass;
+	GObjectClass       *object_class = G_OBJECT_CLASS (class);
 
 	gl_debug (DEBUG_LABEL, "START");
 
-	parent_class = g_type_class_peek_parent (klass);
+	gl_label_object_parent_class = g_type_class_peek_parent (class);
 
-	gobject_class->finalize = gl_label_object_finalize;
+	object_class->finalize = gl_label_object_finalize;
 
-	object_class->set_size = set_size;
-	object_class->get_size = get_size;
+	class->set_size = set_size;
 
 	signals[CHANGED] =
 		g_signal_new ("changed",
-			      G_OBJECT_CLASS_TYPE (gobject_class),
+			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (glLabelObjectClass, changed),
 			      NULL, NULL,
@@ -155,7 +125,7 @@ gl_label_object_class_init (glLabelObjectClass *klass)
 
 	signals[MOVED] =
 		g_signal_new ("moved",
-			      G_OBJECT_CLASS_TYPE (gobject_class),
+			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (glLabelObjectClass, moved),
 			      NULL, NULL,
@@ -164,7 +134,7 @@ gl_label_object_class_init (glLabelObjectClass *klass)
 			      2, G_TYPE_DOUBLE, G_TYPE_DOUBLE);
 	signals[FLIP_ROTATE] =
 		g_signal_new ("flip_rotate",
-			      G_OBJECT_CLASS_TYPE (gobject_class),
+			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (glLabelObjectClass, flip_rotate),
 			      NULL, NULL,
@@ -173,7 +143,7 @@ gl_label_object_class_init (glLabelObjectClass *klass)
 			      0);
 	signals[TOP] =
 		g_signal_new ("top",
-			      G_OBJECT_CLASS_TYPE (gobject_class),
+			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (glLabelObjectClass, top),
 			      NULL, NULL,
@@ -183,33 +153,42 @@ gl_label_object_class_init (glLabelObjectClass *klass)
 
 	signals[BOTTOM] =
 		g_signal_new ("bottom",
-			      G_OBJECT_CLASS_TYPE (gobject_class),
+			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (glLabelObjectClass, bottom),
 			      NULL, NULL,
 			      gl_marshal_VOID__VOID,
 			      G_TYPE_NONE,
 			      0);
+	signals[REMOVED] =
+		g_signal_new ("removed",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (glLabelObjectClass, removed),
+			      NULL, NULL,
+			      gl_marshal_VOID__VOID,
+			      G_TYPE_NONE,
+			      0);
 
 	gl_debug (DEBUG_LABEL, "END");
 }
 
 static void
-gl_label_object_instance_init (glLabelObject *object)
+gl_label_object_init (glLabelObject *object)
 {
 	gl_debug (DEBUG_LABEL, "START");
 
-	object->private = g_new0 (glLabelObjectPrivate, 1);
+	object->priv = g_new0 (glLabelObjectPrivate, 1);
 
-	object->private->name = g_strdup_printf ("object%d", instance++);
+	object->priv->name = g_strdup_printf ("object%d", instance++);
 
-	art_affine_identity (object->private->affine);
+	cairo_matrix_init_identity (&object->priv->matrix);
 
-	object->private->shadow_state = FALSE;
-	object->private->shadow_x = DEFAULT_SHADOW_X_OFFSET;
-	object->private->shadow_y = DEFAULT_SHADOW_Y_OFFSET;
-	object->private->shadow_color_node = gl_color_node_new_default ();
-	object->private->shadow_opacity = DEFAULT_SHADOW_OPACITY;
+	object->priv->shadow_state = FALSE;
+	object->priv->shadow_x = DEFAULT_SHADOW_X_OFFSET;
+	object->priv->shadow_y = DEFAULT_SHADOW_Y_OFFSET;
+	object->priv->shadow_color_node = gl_color_node_new_default ();
+	object->priv->shadow_opacity = DEFAULT_SHADOW_OPACITY;
 
 	gl_debug (DEBUG_LABEL, "END");
 }
@@ -217,19 +196,16 @@ gl_label_object_instance_init (glLabelObject *object)
 static void
 gl_label_object_finalize (GObject *object)
 {
-	glLabel       *parent;
+        glLabelObject *label_object = GL_LABEL_OBJECT (object);
 
 	gl_debug (DEBUG_LABEL, "START");
 
 	g_return_if_fail (object && GL_IS_LABEL_OBJECT (object));
 
-	parent = GL_LABEL_OBJECT(object)->parent;
-	gl_label_remove_object (parent, GL_LABEL_OBJECT(object));
+	g_free (label_object->priv->name);
+	g_free (label_object->priv);
 
-	g_free (GL_LABEL_OBJECT(object)->private->name);
-	g_free (GL_LABEL_OBJECT(object)->private);
-
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (gl_label_object_parent_class)->finalize (object);
 
 	gl_debug (DEBUG_LABEL, "END");
 }
@@ -262,7 +238,7 @@ gl_label_object_dup (glLabelObject *src_object,
 {
 	glLabelObject    *dst_object;
 	gdouble           x, y, w, h;
-	gdouble           affine[6];
+        cairo_matrix_t    matrix;
 	gboolean          shadow_state;
 	gdouble           shadow_x, shadow_y;
 	glColorNode      *shadow_color_node;
@@ -270,7 +246,7 @@ gl_label_object_dup (glLabelObject *src_object,
 
 	gl_debug (DEBUG_LABEL, "START");
 
-	g_return_if_fail (src_object && GL_IS_LABEL_OBJECT (src_object));
+	g_return_val_if_fail (src_object && GL_IS_LABEL_OBJECT (src_object), NULL);
 
 	dst_object = g_object_new (G_OBJECT_TYPE(src_object), NULL);
 
@@ -278,7 +254,7 @@ gl_label_object_dup (glLabelObject *src_object,
 
 	gl_label_object_get_position      (src_object, &x, &y);
 	gl_label_object_get_size          (src_object, &w, &h);
-	gl_label_object_get_affine        (src_object, affine);
+	gl_label_object_get_matrix        (src_object, &matrix);
 	gl_label_object_get_shadow_offset (src_object, &shadow_x, &shadow_y);
 	shadow_color_node = gl_label_object_get_shadow_color   (src_object);
 	shadow_opacity    = gl_label_object_get_shadow_opacity (src_object);
@@ -286,7 +262,7 @@ gl_label_object_dup (glLabelObject *src_object,
 
 	gl_label_object_set_position (dst_object, x, y);
 	gl_label_object_set_size     (dst_object, w, h);
-	gl_label_object_set_affine   (dst_object, affine);
+	gl_label_object_set_matrix   (dst_object, &matrix);
 	gl_label_object_set_shadow_offset  (dst_object, shadow_x, shadow_y);
 	gl_label_object_set_shadow_color   (dst_object, shadow_color_node);
 	gl_label_object_set_shadow_opacity (dst_object, shadow_opacity);
@@ -360,13 +336,41 @@ gl_label_object_get_parent (glLabelObject *object)
 {
 	gl_debug (DEBUG_LABEL, "START");
 
-	g_return_if_fail (object && GL_IS_LABEL_OBJECT (object));
+	g_return_val_if_fail (object && GL_IS_LABEL_OBJECT (object), NULL);
 
 	gl_debug (DEBUG_LABEL, "END");
 
 	return object->parent;
 }
 
+/*****************************************************************************/
+/* Set remove object from parent.                                            */
+/*****************************************************************************/
+void
+gl_label_object_remove (glLabelObject *object)
+{
+	glLabel *parent;
+
+	gl_debug (DEBUG_LABEL, "START");
+
+	g_return_if_fail (object && GL_IS_LABEL_OBJECT (object));
+
+	parent = object->parent;
+	if ( parent != NULL ) {
+		g_signal_handlers_disconnect_by_func (parent,
+						      G_CALLBACK(merge_changed_cb),
+						      object);
+		gl_label_remove_object (parent, object);
+
+                g_signal_emit (G_OBJECT(object), signals[REMOVED], 0);
+
+                g_object_unref (G_OBJECT(object));
+	}
+
+
+	gl_debug (DEBUG_LABEL, "END");
+}
+
 /*****************************************************************************/
 /* Set name of object.                                                       */
 /*****************************************************************************/
@@ -378,8 +382,8 @@ gl_label_object_set_name (glLabelObject *object,
 
 	g_return_if_fail (object && GL_IS_LABEL_OBJECT (object));
 
-	g_free(object->private->name);
-	object->private->name = name;
+	g_free(object->priv->name);
+	object->priv->name = name;
 
 	g_signal_emit (G_OBJECT(object), signals[CHANGED], 0);
 
@@ -394,11 +398,11 @@ gl_label_object_get_name (glLabelObject *object)
 {
 	gl_debug (DEBUG_LABEL, "START");
 
-	g_return_if_fail (object && GL_IS_LABEL_OBJECT (object));
+	g_return_val_if_fail (object && GL_IS_LABEL_OBJECT (object), NULL);
 
 	gl_debug (DEBUG_LABEL, "END");
 
-	return g_strdup(object->private->name);
+	return g_strdup(object->priv->name);
 }
 
 /*****************************************************************************/
@@ -415,13 +419,13 @@ gl_label_object_set_position (glLabelObject *object,
 
 	g_return_if_fail (object && GL_IS_LABEL_OBJECT (object));
 
-	if ( (x != object->private->x) || (y != object->private->y) ) {
+	if ( (x != object->priv->x) || (y != object->priv->y) ) {
 
-		dx = x - object->private->x;
-		dy = y - object->private->y;
+		dx = x - object->priv->x;
+		dy = y - object->priv->y;
 
-		object->private->x = x;
-		object->private->y = y;
+		object->priv->x = x;
+		object->priv->y = y;
 
 		g_signal_emit (G_OBJECT(object), signals[MOVED], 0, dx, dy);
 
@@ -444,12 +448,12 @@ gl_label_object_set_position_relative (glLabelObject *object,
 
 	if ( (dx != 0.0) || (dy != 0.0) ) {
 
-		object->private->x += dx;
-		object->private->y += dy;
+		object->priv->x += dx;
+		object->priv->y += dy;
 
 		gl_debug (DEBUG_LABEL, "       x = %f, y= %f",
-			  object->private->x,
-			  object->private->y);
+			  object->priv->x,
+			  object->priv->y);
 
 		g_signal_emit (G_OBJECT(object), signals[MOVED], 0, dx, dy);
 
@@ -470,8 +474,8 @@ gl_label_object_get_position (glLabelObject *object,
 
 	g_return_if_fail (object && GL_IS_LABEL_OBJECT (object));
 
-	*x = object->private->x;
-	*y = object->private->y;
+	*x = object->priv->x;
+	*y = object->priv->y;
 
 	gl_debug (DEBUG_LABEL, "END");
 }
@@ -486,10 +490,10 @@ set_size (glLabelObject *object,
 {
 	g_return_if_fail (object && GL_IS_LABEL_OBJECT (object));
 
-	if ( (object->private->w != w) || (object->private->h != h) ) {
+	if ( (object->priv->w != w) || (object->priv->h != h) ) {
 
-		object->private->w = w;
-		object->private->h = h;
+		object->priv->w = w;
+		object->priv->h = h;
 
 		g_signal_emit (G_OBJECT(object), signals[CHANGED], 0);
 	}
@@ -512,7 +516,7 @@ gl_label_object_set_size (glLabelObject *object,
 		/* We have an object specific method, use it */
 		GL_LABEL_OBJECT_GET_CLASS(object)->set_size (object, w, h);
 
-		object->private->aspect_ratio = h / w;
+		object->priv->aspect_ratio = h / w;
 
 	}
 
@@ -532,13 +536,13 @@ gl_label_object_set_size_honor_aspect (glLabelObject *object,
 
 	g_return_if_fail (object && GL_IS_LABEL_OBJECT (object));
 
-	if ( h > w*object->private->aspect_ratio ) {
+	if ( h > w*object->priv->aspect_ratio ) {
 
-		h = w * object->private->aspect_ratio;
+		h = w * object->priv->aspect_ratio;
 
 	} else {
 
-		w = h / object->private->aspect_ratio;
+		w = h / object->priv->aspect_ratio;
 
 	}
 
@@ -552,18 +556,18 @@ gl_label_object_set_size_honor_aspect (glLabelObject *object,
 	gl_debug (DEBUG_LABEL, "END");
 }
 
-/*---------------------------------------------------------------------------*/
-/* PRIVATE.  Default get size method.                                        */
-/*---------------------------------------------------------------------------*/
-static void
-get_size (glLabelObject *object,
-	  gdouble       *w,
-	  gdouble       *h)
+/*****************************************************************************/
+/* Get raw size method (don't let object content adjust size).               */
+/*****************************************************************************/
+void
+gl_label_object_get_raw_size (glLabelObject *object,
+                              gdouble       *w,
+                              gdouble       *h)
 {
 	g_return_if_fail (object && GL_IS_LABEL_OBJECT (object));
 
-	*w = object->private->w;
-	*h = object->private->h;
+	*w = object->priv->w;
+	*h = object->priv->h;
 }
 
 /*****************************************************************************/
@@ -585,7 +589,7 @@ gl_label_object_get_size (glLabelObject *object,
 
 	} else {
 
-		get_size (object, w, h);
+		gl_label_object_get_raw_size (object, w, h);
 
 	}
 
@@ -597,44 +601,43 @@ gl_label_object_get_size (glLabelObject *object,
 /*****************************************************************************/
 void
 gl_label_object_get_extent (glLabelObject *object,
-			    gdouble       *x1,
-			    gdouble       *y1,
-			    gdouble       *x2,
-			    gdouble       *y2)
+                            glLabelRegion *region)
 {
-	gdouble  w, h;
-	ArtPoint a1, a2, a3, a4, b1, b2, b3, b4;
-	gdouble  affine[6];
+	gdouble        w, h;
+        gdouble        line_w;
+	gdouble        xa1, ya1, xa2, ya2, xa3, ya3, xa4, ya4;
+        cairo_matrix_t matrix;
 
 	gl_debug (DEBUG_LABEL, "START");
 
 	g_return_if_fail (object && GL_IS_LABEL_OBJECT (object));
 
 	gl_label_object_get_size (object, &w, &h);
+        line_w = gl_label_object_get_line_width (object);
 
-	/* setup untransformed corners of bounding box */
-	a1.x = 0.0;
-	a1.y = 0.0;
-	a2.x = w;
-	a2.y = 0.0;
-	a3.x = w;
-	a3.y = h;
-	a4.x = 0.0;
-	a4.y = h;
+	/* setup untransformed corners of bounding box, account for line width */
+	xa1 =   - line_w/2;
+	ya1 =   - line_w/2;
+	xa2 = w + line_w/2;
+	ya2 =   - line_w/2;
+	xa3 = w + line_w/2;
+	ya3 = h + line_w/2;
+	xa4 =   - line_w/2;
+	ya4 = h + line_w/2;
 
 	/* transform these points */
-	gl_label_object_get_affine (object, affine);
-	art_affine_point (&b1, &a1, affine);
-	art_affine_point (&b2, &a2, affine);
-	art_affine_point (&b3, &a3, affine);
-	art_affine_point (&b4, &a4, affine);
+	gl_label_object_get_matrix (object, &matrix);
+        cairo_matrix_transform_point (&matrix, &xa1, &ya1);
+        cairo_matrix_transform_point (&matrix, &xa2, &ya2);
+        cairo_matrix_transform_point (&matrix, &xa3, &ya3);
+        cairo_matrix_transform_point (&matrix, &xa4, &ya4);
 
 	/* now find the maximum extent of these points in x and y */
-	*x1 = MIN (b1.x, MIN (b2.x, MIN (b3.x, b4.x))) + object->private->x;
-	*y1 = MIN (b1.y, MIN (b2.y, MIN (b3.y, b4.y))) + object->private->y;
-	*x2 = MAX (b1.x, MAX (b2.x, MAX (b3.x, b4.x))) + object->private->x;
-	*y2 = MAX (b1.y, MAX (b2.y, MAX (b3.y, b4.y))) + object->private->y;
-	
+	region->x1 = MIN (xa1, MIN (xa2, MIN (xa3, xa4))) + object->priv->x;
+	region->y1 = MIN (ya1, MIN (ya2, MIN (ya3, ya4))) + object->priv->y;
+	region->x2 = MAX (xa1, MAX (xa2, MAX (xa3, xa4))) + object->priv->x;
+	region->y2 = MAX (ya1, MAX (ya2, MAX (ya3, ya4))) + object->priv->y;
+
 	gl_debug (DEBUG_LABEL, "END");
 }
 
@@ -707,7 +710,7 @@ gl_label_object_set_font_size (glLabelObject     *object,
 /****************************************************************************/
 void
 gl_label_object_set_font_weight (glLabelObject     *object,
-				 GnomeFontWeight    font_weight)
+				 PangoWeight        font_weight)
 {
 	gl_debug (DEBUG_LABEL, "START");
 
@@ -750,7 +753,7 @@ gl_label_object_set_font_italic_flag (glLabelObject     *object,
 /****************************************************************************/
 void
 gl_label_object_set_text_alignment (glLabelObject     *object,
-				    GtkJustification   text_alignment)
+				    PangoAlignment     text_alignment)
 {
 	gl_debug (DEBUG_LABEL, "START");
 
@@ -860,14 +863,14 @@ gl_label_object_get_font_size (glLabelObject     *object)
 /****************************************************************************/
 /* Get font weight for all text contained in object.                        */
 /****************************************************************************/
-GnomeFontWeight
+PangoWeight    
 gl_label_object_get_font_weight (glLabelObject     *object)
 {
-	GnomeFontWeight ret = GNOME_FONT_BOOK;
+	PangoWeight     ret = PANGO_WEIGHT_NORMAL;
 
 	gl_debug (DEBUG_LABEL, "START");
 
-	g_return_val_if_fail (object && GL_IS_LABEL_OBJECT (object), GNOME_FONT_BOOK);
+	g_return_val_if_fail (object && GL_IS_LABEL_OBJECT (object), PANGO_WEIGHT_NORMAL);
 
 	if ( GL_LABEL_OBJECT_GET_CLASS(object)->get_font_weight != NULL ) {
 
@@ -908,14 +911,14 @@ gl_label_object_get_font_italic_flag (glLabelObject     *object)
 /****************************************************************************/
 /* Get text alignment for all text contained in object.                     */
 /****************************************************************************/
-GtkJustification
+PangoAlignment
 gl_label_object_get_text_alignment (glLabelObject     *object)
 {
-	GtkJustification ret = GTK_JUSTIFY_LEFT;
+	PangoAlignment ret = PANGO_ALIGN_LEFT;
 
 	gl_debug (DEBUG_LABEL, "START");
 
-	g_return_val_if_fail (object && GL_IS_LABEL_OBJECT (object), GTK_JUSTIFY_LEFT);
+	g_return_val_if_fail (object && GL_IS_LABEL_OBJECT (object), PANGO_ALIGN_LEFT);
 
 	if ( GL_LABEL_OBJECT_GET_CLASS(object)->get_text_alignment != NULL ) {
 
@@ -1189,9 +1192,9 @@ gl_label_object_set_shadow_state (glLabelObject     *object,
 
 	g_return_if_fail (object && GL_IS_LABEL_OBJECT (object));
 
-	if (object->private->shadow_state != state)
+	if (object->priv->shadow_state != state)
 	{
-		object->private->shadow_state = state;
+		object->priv->shadow_state = state;
 		gl_label_object_emit_changed (object);
 	}
 
@@ -1208,7 +1211,7 @@ gl_label_object_get_shadow_state (glLabelObject     *object)
 
 	g_return_val_if_fail (object && GL_IS_LABEL_OBJECT (object), FALSE);
 
-	return object->private->shadow_state;
+	return object->priv->shadow_state;
 }
 
 /****************************************************************************/
@@ -1223,10 +1226,10 @@ gl_label_object_set_shadow_offset (glLabelObject     *object,
 
 	g_return_if_fail (object && GL_IS_LABEL_OBJECT (object));
 
-	if ( (x != object->private->shadow_x) || (y != object->private->shadow_y) )
+	if ( (x != object->priv->shadow_x) || (y != object->priv->shadow_y) )
 	{
-		object->private->shadow_x = x;
-		object->private->shadow_y = y;
+		object->priv->shadow_x = x;
+		object->priv->shadow_y = y;
 
 		gl_label_object_emit_changed (object);
 	}
@@ -1246,8 +1249,8 @@ gl_label_object_get_shadow_offset (glLabelObject     *object,
 
 	g_return_if_fail (object && GL_IS_LABEL_OBJECT (object));
 
-	*x = object->private->shadow_x;
-	*y = object->private->shadow_y;
+	*x = object->priv->shadow_x;
+	*y = object->priv->shadow_y;
 
 	gl_debug (DEBUG_LABEL, "END");
 }
@@ -1263,10 +1266,10 @@ gl_label_object_set_shadow_color (glLabelObject     *object,
 
 	g_return_if_fail (object && GL_IS_LABEL_OBJECT (object));
 
-	if ( !gl_color_node_equal (object->private->shadow_color_node, color_node ))
+	if ( !gl_color_node_equal (object->priv->shadow_color_node, color_node ))
 	{
-		gl_color_node_free (&(object->private->shadow_color_node));
-		object->private->shadow_color_node = gl_color_node_dup (color_node);
+		gl_color_node_free (&(object->priv->shadow_color_node));
+		object->priv->shadow_color_node = gl_color_node_dup (color_node);
 		gl_label_object_emit_changed (GL_LABEL_OBJECT(object));
 	}
 
@@ -1283,7 +1286,7 @@ gl_label_object_get_shadow_color (glLabelObject     *object)
 
 	g_return_val_if_fail (object && GL_IS_LABEL_OBJECT (object), NULL);
 
-	return gl_color_node_dup (object->private->shadow_color_node);
+	return gl_color_node_dup (object->priv->shadow_color_node);
 }
 
 /****************************************************************************/
@@ -1297,9 +1300,9 @@ gl_label_object_set_shadow_opacity (glLabelObject     *object,
 
 	g_return_if_fail (object && GL_IS_LABEL_OBJECT (object));
 
-	if (object->private->shadow_opacity != alpha)
+	if (object->priv->shadow_opacity != alpha)
 	{
-		object->private->shadow_opacity = alpha;
+		object->priv->shadow_opacity = alpha;
 		gl_label_object_emit_changed (object);
 	}
 
@@ -1316,7 +1319,7 @@ gl_label_object_get_shadow_opacity (glLabelObject     *object)
 
 	g_return_val_if_fail (object && GL_IS_LABEL_OBJECT (object), FALSE);
 
-	return object->private->shadow_opacity;
+	return object->priv->shadow_opacity;
 }
 
 /****************************************************************************/
@@ -1325,14 +1328,14 @@ gl_label_object_get_shadow_opacity (glLabelObject     *object)
 void
 gl_label_object_flip_horiz (glLabelObject *object)
 {
-	gdouble flip_affine[6];
+        cairo_matrix_t flip_matrix;
 
 	gl_debug (DEBUG_LABEL, "START");
 
 	g_return_if_fail (object && GL_IS_LABEL_OBJECT (object));
 
-	art_affine_scale (flip_affine, -1.0, 1.0);
-	art_affine_multiply (object->private->affine, object->private->affine, flip_affine);
+        cairo_matrix_init_scale (&flip_matrix, -1.0, 1.0);
+        cairo_matrix_multiply (&object->priv->matrix, &object->priv->matrix, &flip_matrix);
 
 	g_signal_emit (G_OBJECT(object), signals[FLIP_ROTATE], 0);
 
@@ -1345,14 +1348,14 @@ gl_label_object_flip_horiz (glLabelObject *object)
 void
 gl_label_object_flip_vert (glLabelObject *object)
 {
-	gdouble flip_affine[6];
+        cairo_matrix_t flip_matrix;
 
 	gl_debug (DEBUG_LABEL, "START");
 
 	g_return_if_fail (object && GL_IS_LABEL_OBJECT (object));
 
-	art_affine_scale (flip_affine, 1.0, -1.0);
-	art_affine_multiply (object->private->affine, object->private->affine, flip_affine);
+        cairo_matrix_init_scale (&flip_matrix, 1.0, -1.0);
+        cairo_matrix_multiply (&object->priv->matrix, &object->priv->matrix, &flip_matrix);
 
 	g_signal_emit (G_OBJECT(object), signals[FLIP_ROTATE], 0);
 
@@ -1366,14 +1369,14 @@ void
 gl_label_object_rotate (glLabelObject *object,
 			gdouble        theta_degs)
 {
-	gdouble rotate_affine[6];
+        cairo_matrix_t rotate_matrix;
 
 	gl_debug (DEBUG_LABEL, "START");
 
 	g_return_if_fail (object && GL_IS_LABEL_OBJECT (object));
 
-	art_affine_rotate (rotate_affine, theta_degs);
-	art_affine_multiply (object->private->affine, object->private->affine, rotate_affine);
+        cairo_matrix_init_rotate (&rotate_matrix, theta_degs*(G_PI/180.));
+        cairo_matrix_multiply (&object->priv->matrix, &object->priv->matrix, &rotate_matrix);
 
 	g_signal_emit (G_OBJECT(object), signals[FLIP_ROTATE], 0);
 
@@ -1384,75 +1387,28 @@ gl_label_object_rotate (glLabelObject *object,
 /* Set raw affine                                                           */
 /****************************************************************************/
 void
-gl_label_object_set_affine (glLabelObject *object,
-			    gdouble        affine[6])
+gl_label_object_set_matrix (glLabelObject  *object,
+                            cairo_matrix_t *matrix)
 {
 	gl_debug (DEBUG_LABEL, "");
 
 	g_return_if_fail (object && GL_IS_LABEL_OBJECT (object));
 
-	object->private->affine[0] = affine[0];
-	object->private->affine[1] = affine[1];
-	object->private->affine[2] = affine[2];
-	object->private->affine[3] = affine[3];
-	object->private->affine[4] = affine[4];
-	object->private->affine[5] = affine[5];
+        object->priv->matrix = *matrix;
 }
 
 /****************************************************************************/
 /* Get raw affine                                                           */
 /****************************************************************************/
 void
-gl_label_object_get_affine (glLabelObject *object,
-			    gdouble        affine[6])
+gl_label_object_get_matrix (glLabelObject  *object,
+                            cairo_matrix_t *matrix)
 {
 	gl_debug (DEBUG_LABEL, "");
 
 	g_return_if_fail (object && GL_IS_LABEL_OBJECT (object));
 
-	affine[0] = object->private->affine[0];
-	affine[1] = object->private->affine[1];
-	affine[2] = object->private->affine[2];
-	affine[3] = object->private->affine[3];
-	affine[4] = object->private->affine[4];
-	affine[5] = object->private->affine[5];
-}
-
-/****************************************************************************/
-/* Get i2w affine, i.e. applied affine + translation.                       */
-/****************************************************************************/
-void
-gl_label_object_get_i2w_affine (glLabelObject *object,
-				gdouble        affine[6])
-{
-	gdouble x, y;
-	gdouble translation[6];
-
-	gl_debug (DEBUG_LABEL, "");
-
-	g_return_if_fail (object && GL_IS_LABEL_OBJECT (object));
-
-	gl_label_object_get_affine (object, affine);
-	gl_label_object_get_position (object, &x, &y);
-
-	art_affine_translate (translation, x, y);
-	art_affine_multiply (affine, affine, translation);
-}
-
-/****************************************************************************/
-/* Get w2i affine, i.e. inverse of applied affine + translation.            */
-/****************************************************************************/
-void
-gl_label_object_get_w2i_affine (glLabelObject *object,
-				gdouble        affine[6])
-{
-	gdouble i2w[6];
-	gl_debug (DEBUG_LABEL, "");
-
-	g_return_if_fail (object && GL_IS_LABEL_OBJECT (object));
-
-	gl_label_object_get_i2w_affine (object, i2w);
-	art_affine_invert (affine, i2w);
+        *matrix = object->priv->matrix;
 }
 
 /****************************************************************************/
@@ -1502,3 +1458,68 @@ merge_changed_cb (glLabel       *label,
 {
 	gl_label_object_emit_changed (object);
 }
+
+/*****************************************************************************/
+/* Draw object                                                               */
+/*****************************************************************************/
+void
+gl_label_object_draw (glLabelObject *object,
+                      cairo_t       *cr,
+                      gboolean       screen_flag,
+                      glMergeRecord *record)
+
+{
+        gdouble        x0, y0;
+        cairo_matrix_t matrix;
+	gboolean       shadow_state;
+	gdouble        shadow_x, shadow_y;
+
+	gl_debug (DEBUG_LABEL, "START");
+
+	g_return_if_fail (object && GL_IS_LABEL_OBJECT (object));
+
+        gl_label_object_get_position (object, &x0, &y0);
+        gl_label_object_get_matrix (object, &matrix);
+
+        cairo_save (cr);
+        cairo_translate (cr, x0, y0);
+
+        if ( GL_LABEL_OBJECT_GET_CLASS(object)->draw_shadow != NULL ) {
+
+                shadow_state = gl_label_object_get_shadow_state (object);
+
+                if ( shadow_state )
+                {
+                        gl_label_object_get_shadow_offset (object, &shadow_x, &shadow_y);
+
+                        cairo_save (cr);
+                        cairo_translate (cr, shadow_x, shadow_y);
+                        cairo_transform (cr, &matrix);
+
+                        GL_LABEL_OBJECT_GET_CLASS(object)->draw_shadow (object,
+                                                                        cr,
+                                                                        screen_flag,
+                                                                        record);
+
+                        cairo_restore (cr);
+                }
+        }
+
+        if ( GL_LABEL_OBJECT_GET_CLASS(object)->draw_object != NULL ) {
+
+                cairo_save (cr);
+                cairo_transform (cr, &matrix);
+
+                GL_LABEL_OBJECT_GET_CLASS(object)->draw_object (object,
+                                                                cr,
+                                                                screen_flag,
+                                                                record);
+
+                cairo_restore (cr);
+        }
+
+        cairo_restore (cr);
+
+	gl_debug (DEBUG_LABEL, "END");
+}
+
