@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
 
 /*
  *  (GLABELS) Label and Business Card Creation program for GNOME
@@ -31,9 +31,11 @@
 #include <gtk/gtkcombobox.h>
 #include <gtk/gtkstock.h>
 #include <gtk/gtktreestore.h>
+#include <gtk/gtktreeselection.h>
 #include <gtk/gtkcellrenderertoggle.h>
 #include <gtk/gtkcellrenderertext.h>
 #include <gtk/gtklabel.h>
+#include <string.h>
 
 #include "label.h"
 #include "merge.h"
@@ -48,8 +50,6 @@
 
 struct _glMergePropertiesDialogPrivate {
 
-	GladeXML     *gui;
-
 	glLabel      *label;
 	glMerge      *merge;
 
@@ -84,14 +84,11 @@ enum {
 /* Private globals                           */
 /*===========================================*/
 
-static GtkDialogClass* parent_class = NULL;
 
 /*===========================================*/
 /* Local function prototypes                 */
 /*===========================================*/
 
-static void gl_merge_properties_dialog_class_init (glMergePropertiesDialogClass *klass);
-static void gl_merge_properties_dialog_init       (glMergePropertiesDialog      *dlg);
 static void gl_merge_properties_dialog_finalize   (GObject                      *object);
 static void gl_merge_properties_dialog_construct  (glMergePropertiesDialog      *dialog,
 						   glLabel                      *label,
@@ -124,42 +121,16 @@ static void unselect_all_button_clicked_cb        (GtkWidget
 /*****************************************************************************/
 /* Boilerplate object stuff.                                                 */
 /*****************************************************************************/
-GType
-gl_merge_properties_dialog_get_type (void)
-{
-	static GType type = 0;
-
-	if (!type)
-    	{
-      		static const GTypeInfo info =
-      		{
-			sizeof (glMergePropertiesDialogClass),
-        		NULL,		/* base_init */
-        		NULL,		/* base_finalize */
-        		(GClassInitFunc) gl_merge_properties_dialog_class_init,
-        		NULL,           /* class_finalize */
-        		NULL,           /* class_data */
-        		sizeof (glMergePropertiesDialog),
-        		0,              /* n_preallocs */
-        		(GInstanceInitFunc) gl_merge_properties_dialog_init,
-			NULL
-      		};
-
-     		type = g_type_register_static (GTK_TYPE_DIALOG,
-					       "glMergePropertiesDialog", &info, 0);
-    	}
-
-	return type;
-}
+G_DEFINE_TYPE (glMergePropertiesDialog, gl_merge_properties_dialog, GTK_TYPE_DIALOG);
 
 static void
-gl_merge_properties_dialog_class_init (glMergePropertiesDialogClass *klass)
+gl_merge_properties_dialog_class_init (glMergePropertiesDialogClass *class)
 {
-	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	GObjectClass *object_class = G_OBJECT_CLASS (class);
 
 	gl_debug (DEBUG_MERGE, "");
 	
-  	parent_class = g_type_class_peek_parent (klass);
+  	gl_merge_properties_dialog_parent_class = g_type_class_peek_parent (class);
 
   	object_class->finalize = gl_merge_properties_dialog_finalize;  	
 }
@@ -171,15 +142,6 @@ gl_merge_properties_dialog_init (glMergePropertiesDialog *dialog)
 
 	dialog->priv = g_new0 (glMergePropertiesDialogPrivate, 1);
 
-	dialog->priv->gui = glade_xml_new (GLABELS_GLADE_DIR "merge-properties-dialog.glade",
-					   "merge_properties_vbox",
-					   NULL);
-
-	if (!dialog->priv->gui) {
-		g_critical ("Could not open merge-properties-dialog.glade. gLabels may not be installed correctly!");
-		return;
-	}
-
 	gtk_container_set_border_width (GTK_CONTAINER(dialog), GL_HIG_PAD2);
 
 	gtk_dialog_set_has_separator (GTK_DIALOG(dialog), FALSE);
@@ -200,25 +162,20 @@ gl_merge_properties_dialog_init (glMergePropertiesDialog *dialog)
 static void 
 gl_merge_properties_dialog_finalize (GObject *object)
 {
-	glMergePropertiesDialog* dialog;
+	glMergePropertiesDialog* dialog = GL_MERGE_PROPERTIES_DIALOG (object);
 	
 	gl_debug (DEBUG_MERGE, "START");
 
 	g_return_if_fail (object != NULL);
-	
-   	dialog = GL_MERGE_PROPERTIES_DIALOG (object);
-
 	g_return_if_fail (GL_IS_MERGE_PROPERTIES_DIALOG (dialog));
 	g_return_if_fail (dialog->priv != NULL);
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
-
 	if (dialog->priv->merge != NULL) {
 		g_object_unref (G_OBJECT (dialog->priv->merge));
 	}
-
 	g_free (dialog->priv);
 
+	G_OBJECT_CLASS (gl_merge_properties_dialog_parent_class)->finalize (object);
 	gl_debug (DEBUG_MERGE, "END");
 }
 
@@ -251,6 +208,7 @@ gl_merge_properties_dialog_construct (glMergePropertiesDialog *dialog,
 				      glLabel                 *label,
 				      GtkWindow               *window)
 {
+	GladeXML          *gui;
 	gchar             *description;
 	glMergeSrcType     src_type;
 	gchar             *src;
@@ -271,21 +229,25 @@ gl_merge_properties_dialog_construct (glMergePropertiesDialog *dialog,
 		gtk_window_set_destroy_with_parent (GTK_WINDOW(dialog), TRUE);
 	}
 
-	vbox = glade_xml_get_widget (dialog->priv->gui,
-				     "merge_properties_vbox");
+	gui = glade_xml_new (GLABELS_GLADE_DIR "merge-properties-dialog.glade",
+                             "merge_properties_vbox", NULL);
+
+	if (!gui) {
+		g_critical ("Could not open merge-properties-dialog.glade. gLabels may not be installed correctly!");
+		return;
+	}
+
+	vbox = glade_xml_get_widget (gui, "merge_properties_vbox");
 	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), vbox);
 
-	dialog->priv->type_combo    = glade_xml_get_widget (dialog->priv->gui,
-							    "type_combo");
-	dialog->priv->location_vbox = glade_xml_get_widget (dialog->priv->gui,
-							    "location_vbox");
-	dialog->priv->treeview      = glade_xml_get_widget (dialog->priv->gui,
-							    "treeview");
+	dialog->priv->type_combo    = glade_xml_get_widget (gui, "type_combo");
+	dialog->priv->location_vbox = glade_xml_get_widget (gui, "location_vbox");
+	dialog->priv->treeview      = glade_xml_get_widget (gui, "treeview");
 
-	dialog->priv->select_all_button   = glade_xml_get_widget (dialog->priv->gui,
-								"select_all_button");
-	dialog->priv->unselect_all_button = glade_xml_get_widget (dialog->priv->gui,
-								"unselect_all_button");
+	dialog->priv->select_all_button   = glade_xml_get_widget (gui, "select_all_button");
+	dialog->priv->unselect_all_button = glade_xml_get_widget (gui, "unselect_all_button");
+
+        g_object_unref (gui);
 
 	gl_util_combo_box_add_text_model (GTK_COMBO_BOX (dialog->priv->type_combo));
 
@@ -409,7 +371,6 @@ type_changed_cb (GtkWidget               *widget,
 	gchar             *name;
 	gchar             *src;
 	glMergeSrcType     src_type;
-	GtkWidget         *wentry;
 
 	gl_debug (DEBUG_MERGE, "START");
 
@@ -477,7 +438,6 @@ src_changed_cb (GtkWidget               *widget,
 		glMergePropertiesDialog *dialog)
 {
 	gchar     *src, *orig_src;
-	GtkWidget *wentry;
 
 	gl_debug (DEBUG_MERGE, "START");
 
