@@ -1,3 +1,5 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
+
 /*
  *  (GLABELS) Label and Business Card Creation program for GNOME
  *
@@ -141,6 +143,8 @@ aspect_toggle_cb (glObjectEditor *editor)
         glWdgtChainButton *toggle;
 	gdouble            w, h;
 
+        if (editor->priv->stop_signals) return;
+
 	gl_debug (DEBUG_EDITOR, "START");
 
 	toggle = GL_WDGT_CHAIN_BUTTON (editor->priv->size_aspect_checkbutton);
@@ -166,6 +170,8 @@ w_spin_cb (glObjectEditor *editor)
 	gdouble            w, h;
         glWdgtChainButton *toggle;
 
+        if (editor->priv->stop_signals) return;
+
 	gl_debug (DEBUG_EDITOR, "START");
 
 	toggle = GL_WDGT_CHAIN_BUTTON (editor->priv->size_aspect_checkbutton);
@@ -177,13 +183,9 @@ w_spin_cb (glObjectEditor *editor)
                 h = w * editor->priv->size_aspect_ratio;
                                                                                 
                 /* Update our sibling control, blocking recursion. */
-                g_signal_handlers_block_by_func (G_OBJECT (editor->priv->size_h_spin),
-						 G_CALLBACK (h_spin_cb),
-						 editor);
+                editor->priv->stop_signals = TRUE;
                 gtk_spin_button_set_value (GTK_SPIN_BUTTON (editor->priv->size_h_spin), h);
-                g_signal_handlers_unblock_by_func (G_OBJECT (editor->priv->size_h_spin),
-						   G_CALLBACK (h_spin_cb),
-						   editor);
+                editor->priv->stop_signals = FALSE;
         }
                                                                                 
         /* Emit our "changed" signal */
@@ -202,6 +204,8 @@ h_spin_cb (glObjectEditor *editor)
 	gdouble            w, h;
         glWdgtChainButton *toggle;
 
+        if (editor->priv->stop_signals) return;
+
 	gl_debug (DEBUG_EDITOR, "START");
 
         toggle = GL_WDGT_CHAIN_BUTTON (editor->priv->size_aspect_checkbutton);
@@ -213,13 +217,9 @@ h_spin_cb (glObjectEditor *editor)
                 w = h / editor->priv->size_aspect_ratio;
                                                                                 
                 /* Update our sibling control, blocking recursion. */
-                g_signal_handlers_block_by_func (G_OBJECT (editor->priv->size_w_spin),
-						 G_CALLBACK (w_spin_cb),
-						 editor);
+                editor->priv->stop_signals = TRUE;
                 gtk_spin_button_set_value (GTK_SPIN_BUTTON (editor->priv->size_w_spin), w);
-                g_signal_handlers_unblock_by_func (G_OBJECT (editor->priv->size_w_spin),
-						   G_CALLBACK (w_spin_cb),
-						   editor);
+                editor->priv->stop_signals = FALSE;
         }
                                                                                 
         /* Emit our "changed" signal */
@@ -236,35 +236,33 @@ static void
 size_reset_cb (glObjectEditor *editor)
 {
 	gdouble w_base, h_base;
-	gdouble w_max, h_max;
+	gdouble w_max, h_max, wh_max;
 	gdouble aspect_ratio;
 
+        if (editor->priv->stop_signals) return;
+
 	gl_debug (DEBUG_EDITOR, "START");
 
-	g_signal_handlers_block_by_func (G_OBJECT (editor->priv->size_w_spin),
-					 G_CALLBACK (w_spin_cb),
-					 editor);
-	g_signal_handlers_block_by_func (G_OBJECT (editor->priv->size_h_spin),
-					 G_CALLBACK (h_spin_cb),
-					 editor);
+        editor->priv->stop_signals = TRUE;
 
 	w_base = editor->priv->w_base;
 	h_base = editor->priv->h_base;
 
 	w_max = editor->priv->w_max;
 	h_max = editor->priv->h_max;
+        wh_max = MAX( w_max, h_max );
 
-	if ( (w_base > w_max) || (h_base > h_max) ) {
+	if ( (w_base > wh_max) || (h_base > wh_max) ) {
 
 		aspect_ratio = h_base / w_base;
 
-		if ( h_max > w_max*aspect_ratio ) {
-			w_base = w_max;
-			h_base = w_max * aspect_ratio;
+		if ( aspect_ratio < 1.0 ) {
+			w_base = wh_max;
+			h_base = wh_max * aspect_ratio;
 
 		} else {
-			w_base = h_max / aspect_ratio;
-			h_base = h_max;
+			w_base = wh_max / aspect_ratio;
+			h_base = wh_max;
 		}
 	}
 
@@ -276,16 +274,12 @@ size_reset_cb (glObjectEditor *editor)
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (editor->priv->size_h_spin),
 				   h_base);
 
-	g_signal_handlers_unblock_by_func (G_OBJECT (editor->priv->size_w_spin),
-					   G_CALLBACK (w_spin_cb),
-					   editor);
-	g_signal_handlers_unblock_by_func (G_OBJECT (editor->priv->size_h_spin),
-					   G_CALLBACK (h_spin_cb),
-					   editor);
         /* Emit our "changed" signal */
         g_signal_emit (G_OBJECT (editor), gl_object_editor_signals[CHANGED], 0);
         g_signal_emit (G_OBJECT (editor), gl_object_editor_signals[SIZE_CHANGED], 0);
 
+        editor->priv->stop_signals = FALSE;
+
 	gl_debug (DEBUG_EDITOR, "END");
 }
 
@@ -299,12 +293,7 @@ gl_object_editor_set_size (glObjectEditor      *editor,
 {
 	gl_debug (DEBUG_EDITOR, "START");
 
-	g_signal_handlers_block_by_func (G_OBJECT(editor->priv->size_w_spin),
-					 w_spin_cb,
-					 editor);
-	g_signal_handlers_block_by_func (G_OBJECT(editor->priv->size_h_spin),
-					 h_spin_cb,
-					 editor);
+        editor->priv->stop_signals = TRUE;
 
 	/* save a copy in internal units */
 	editor->priv->w = w;
@@ -320,16 +309,11 @@ gl_object_editor_set_size (glObjectEditor      *editor,
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (editor->priv->size_w_spin), w);
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (editor->priv->size_h_spin), h);
 
-	g_signal_handlers_unblock_by_func (G_OBJECT(editor->priv->size_w_spin),
-					   w_spin_cb,
-					   editor);
-	g_signal_handlers_unblock_by_func (G_OBJECT(editor->priv->size_h_spin),
-					   h_spin_cb,
-					   editor);
-
 	/* Update aspect ratio */
 	editor->priv->size_aspect_ratio = h / w;
 
+        editor->priv->stop_signals = FALSE;
+
 	gl_debug (DEBUG_EDITOR, "END");
 }
 
@@ -342,42 +326,39 @@ gl_object_editor_set_max_size (glObjectEditor      *editor,
 			       gdouble              h_max)
 {
 	gdouble tmp;
+        gdouble wh_max;
 
 	gl_debug (DEBUG_EDITOR, "START");
 
-	g_signal_handlers_block_by_func (G_OBJECT(editor->priv->size_w_spin),
-					 w_spin_cb,
-					 editor);
-	g_signal_handlers_block_by_func (G_OBJECT(editor->priv->size_h_spin),
-					 h_spin_cb,
-					 editor);
+        if (editor->priv->size_page_vbox)
+        {
 
-	/* save a copy in internal units */
-	editor->priv->w_max = w_max;
-	editor->priv->h_max = h_max;
+                editor->priv->stop_signals = TRUE;
 
-	/* convert internal units to displayed units */
-	gl_debug (DEBUG_EDITOR, "internal w_max,h_max = %g, %g", w_max, h_max);
-	w_max *= editor->priv->units_per_point;
-	h_max *= editor->priv->units_per_point;
-	gl_debug (DEBUG_EDITOR, "display w_max,h_max = %g, %g", w_max, h_max);
+                /* save a copy in internal units */
+                editor->priv->w_max = w_max;
+                editor->priv->h_max = h_max;
 
-	/* Set widget values */
-	tmp = gtk_spin_button_get_value (GTK_SPIN_BUTTON (editor->priv->size_w_spin));
-	gtk_spin_button_set_range (GTK_SPIN_BUTTON (editor->priv->size_w_spin),
-				   0.0, 2.0*w_max);
-	gtk_spin_button_set_value (GTK_SPIN_BUTTON (editor->priv->size_w_spin), tmp);
-	tmp = gtk_spin_button_get_value (GTK_SPIN_BUTTON (editor->priv->size_h_spin));
-	gtk_spin_button_set_range (GTK_SPIN_BUTTON (editor->priv->size_h_spin),
-				   0.0, 2.0*h_max);
-	gtk_spin_button_set_value (GTK_SPIN_BUTTON (editor->priv->size_h_spin), tmp);
+                /* convert internal units to displayed units */
+                gl_debug (DEBUG_EDITOR, "internal w_max,h_max = %g, %g", w_max, h_max);
+                w_max *= editor->priv->units_per_point;
+                h_max *= editor->priv->units_per_point;
+                wh_max = MAX( w_max, h_max );
+                gl_debug (DEBUG_EDITOR, "display w_max,h_max = %g, %g", w_max, h_max);
 
-	g_signal_handlers_unblock_by_func (G_OBJECT(editor->priv->size_w_spin),
-					   w_spin_cb,
-					   editor);
-	g_signal_handlers_unblock_by_func (G_OBJECT(editor->priv->size_h_spin),
-					   h_spin_cb,
-					   editor);
+                /* Set widget values */
+                tmp = gtk_spin_button_get_value (GTK_SPIN_BUTTON (editor->priv->size_w_spin));
+                gtk_spin_button_set_range (GTK_SPIN_BUTTON (editor->priv->size_w_spin),
+                                           0.0, 2.0*wh_max);
+                gtk_spin_button_set_value (GTK_SPIN_BUTTON (editor->priv->size_w_spin), tmp);
+                tmp = gtk_spin_button_get_value (GTK_SPIN_BUTTON (editor->priv->size_h_spin));
+                gtk_spin_button_set_range (GTK_SPIN_BUTTON (editor->priv->size_h_spin),
+                                           0.0, 2.0*wh_max);
+                gtk_spin_button_set_value (GTK_SPIN_BUTTON (editor->priv->size_h_spin), tmp);
+
+                editor->priv->stop_signals = FALSE;
+
+        }
 
 	gl_debug (DEBUG_EDITOR, "END");
 }
@@ -442,12 +423,7 @@ size_prefs_changed_cb (glObjectEditor *editor)
         digits = gl_prefs_get_units_precision ();
 
 	/* Update characteristics of w_spin/h_spin */
-	g_signal_handlers_block_by_func (G_OBJECT(editor->priv->size_w_spin),
-					 w_spin_cb,
-					 editor);
-	g_signal_handlers_block_by_func (G_OBJECT(editor->priv->size_h_spin),
-					 h_spin_cb,
-					 editor);
+        editor->priv->stop_signals = TRUE;
 	gtk_spin_button_set_digits (GTK_SPIN_BUTTON(editor->priv->size_w_spin),
 				    digits);
 	gtk_spin_button_set_digits (GTK_SPIN_BUTTON(editor->priv->size_h_spin),
@@ -456,12 +432,7 @@ size_prefs_changed_cb (glObjectEditor *editor)
 					climb_rate, 10.0*climb_rate);
 	gtk_spin_button_set_increments (GTK_SPIN_BUTTON(editor->priv->size_h_spin),
 					climb_rate, 10.0*climb_rate);
-	g_signal_handlers_unblock_by_func (G_OBJECT(editor->priv->size_w_spin),
-					   w_spin_cb,
-					   editor);
-	g_signal_handlers_unblock_by_func (G_OBJECT(editor->priv->size_h_spin),
-					   h_spin_cb,
-					   editor);
+        editor->priv->stop_signals = FALSE;
 
 	/* Update units_labels */
 	gtk_label_set_text (GTK_LABEL(editor->priv->size_w_units_label),
