@@ -576,7 +576,6 @@ gl_object_editor_construct_color_combo (gchar *name,
 	GtkWidget  *color_combo;
 	ColorGroup *cg;
 	gchar      *cg_name;
-	guint       color;
 	GdkColor   *gdk_color;
 	gchar      *no_color;
 
@@ -584,33 +583,32 @@ gl_object_editor_construct_color_combo (gchar *name,
 
 	case 3:
 		cg_name  = "shadow_color_group";
-		color    = GL_COLOR_SHADOW_DEFAULT;
+                gdk_color = gl_color_to_gdk_color (GL_COLOR_SHADOW_DEFAULT);
 		no_color = _("Default");
 		break;
 
 	case 2:
 		cg_name  = "text_color_group";
-		color    = gl_prefs->default_text_color;
+                gdk_color = gl_color_to_gdk_color (gl_prefs->default_text_color);
 		no_color = _("Default");
 		break;
 
 	case 1:
 		cg_name  = "line_color_group";
-		color    = gl_prefs->default_line_color;
+                gdk_color = NULL;
 		no_color = _("No line");
 		break;
 
 	case 0:
 	default:
 		cg_name  = "fill_color_group";
-		color    = gl_prefs->default_fill_color;
+                gdk_color = NULL;
 		no_color = _("No fill");
 		break;
 
 	}
 
 	cg = color_group_fetch (cg_name, NULL);
-        gdk_color = gl_color_to_gdk_color (color);
 	color_combo = color_combo_new (NULL, no_color, gdk_color, cg);
         g_free (gdk_color);
 
