@@ -1,3 +1,5 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
+
 /*
  *  (GLABELS) Label and Business Card Creation program for GNOME
  *
@@ -36,7 +38,7 @@ void          gl_prefs_init                (void);
 
 const gchar  *gl_prefs_get_page_size       (void);
 
-glUnitsType   gl_prefs_get_units           (void);
+lglUnitsType  gl_prefs_get_units           (void);
 
 const gchar  *gl_prefs_get_units_string    (void);
 
@@ -46,6 +48,8 @@ gdouble       gl_prefs_get_units_step_size (void);
 
 gint          gl_prefs_get_units_precision (void);
 
+void          gl_prefs_add_recent_template (const gchar *name);
+
 G_END_DECLS
 
 #endif /* __PREFS_H__ */
