@@ -301,7 +301,6 @@ gl_ui_property_bar_construct (glUIPropertyBar   *property_bar)
         {
 		gtk_combo_box_set_active (GTK_COMBO_BOX (property_bar->priv->font_family_combo), 0);
 	}
-	gl_util_font_family_list_free (family_names);
 
 	g_signal_connect (G_OBJECT (property_bar->priv->font_family_combo),
 			  "changed", G_CALLBACK (font_family_changed_cb), property_bar);
@@ -406,7 +405,6 @@ reset_to_default_properties (glView *view,
 	gl_util_combo_box_set_active_text (GTK_COMBO_BOX (property_bar->priv->font_family_combo),
 					   good_font_family);
 	g_free (good_font_family);
-	gl_util_font_family_list_free (family_names);
 
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON(property_bar->priv->font_size_spin),
 				   view->default_font_size);
@@ -1222,7 +1220,6 @@ gl_ui_property_bar_construct_color_combo (gchar *name,
 	GtkWidget  *color_combo;
 	ColorGroup *cg;
 	gchar      *cg_name;
-	guint       color;
 	GdkColor   *gdk_color;
 	gchar      *no_color;
 	GdkPixbuf  *pixbuf = NULL;
@@ -1231,14 +1228,14 @@ gl_ui_property_bar_construct_color_combo (gchar *name,
 
 	case 0:
 		cg_name  = "text_color_group";
-		color    = gl_prefs->default_text_color;
+                gdk_color = gl_color_to_gdk_color (gl_prefs->default_text_color);
 		no_color = _("Default");
 		pixbuf = gdk_pixbuf_new_from_inline (-1, stock_text_24, FALSE, NULL);
 		break;
 
 	case 2:
 		cg_name  = "line_color_group";
-		color    = gl_prefs->default_line_color;
+                gdk_color = NULL;
 		no_color = _("No line");
 		pixbuf = gdk_pixbuf_new_from_inline (-1, stock_pencil_24, FALSE, NULL);
 		break;
@@ -1246,7 +1243,7 @@ gl_ui_property_bar_construct_color_combo (gchar *name,
 	case 1:
 	default:
 		cg_name  = "fill_color_group";
-		color    = gl_prefs->default_fill_color;
+                gdk_color = NULL;
 		no_color = _("No fill");
 		pixbuf = gdk_pixbuf_new_from_inline (-1, stock_bucket_fill_24, FALSE, NULL);
 		break;
@@ -1254,7 +1251,6 @@ gl_ui_property_bar_construct_color_combo (gchar *name,
 	}
 
 	cg = color_group_fetch (cg_name, NULL);
-        gdk_color = gl_color_to_gdk_color (color);
 	color_combo = color_combo_new (pixbuf, no_color, gdk_color, cg);
         g_free (gdk_color);
 
