@@ -152,7 +152,7 @@ gl_wdgt_print_copies_construct (glWdgtPrintCopies *copies,
 	copies->sheets_radio =
 	    gtk_radio_button_new_with_label (radio_group, _("Sheets:"));
 	gtk_box_pack_start (GTK_BOX(whbox1), copies->sheets_radio, FALSE, FALSE, 0);
-	adjust = gtk_adjustment_new (1, 1.0, 10.0, 1.0, 10.0, 10.0);
+	adjust = gtk_adjustment_new (1, 1.0, 100.0, 1.0, 10.0, 0.0);
 	copies->sheets_spin = gtk_spin_button_new (GTK_ADJUSTMENT (adjust),
 						   1.0, 0);
 	gtk_box_pack_start (GTK_BOX(whbox1), copies->sheets_spin, FALSE, FALSE, 0);
@@ -171,15 +171,12 @@ gl_wdgt_print_copies_construct (glWdgtPrintCopies *copies,
 	    gtk_radio_button_new_with_label (radio_group, _("Labels"));
 	gtk_box_pack_start (GTK_BOX(whbox1), copies->labels_radio, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX(whbox1), gtk_label_new (_("from:")), FALSE, FALSE, 0);
-	adjust = gtk_adjustment_new (1, 1.0, copies->labels_per_sheet,
-				     1.0, 10.0, 10.0);
+	adjust = gtk_adjustment_new (1, 1.0, copies->labels_per_sheet, 1.0, 10.0, 0.0);
 	copies->first_spin = gtk_spin_button_new (GTK_ADJUSTMENT (adjust),
 						  1.0, 0);
 	gtk_box_pack_start (GTK_BOX(whbox1), copies->first_spin, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX(whbox1), gtk_label_new (_("to:")), FALSE, FALSE, 0);
-	adjust = gtk_adjustment_new (copies->labels_per_sheet,
-				     1.0, copies->labels_per_sheet,
-				     1.0, 10.0, 10.0);
+	adjust = gtk_adjustment_new (copies->labels_per_sheet, 1.0, copies->labels_per_sheet, 1.0, 10.0, 0.0);
 	copies->last_spin = gtk_spin_button_new (GTK_ADJUSTMENT (adjust),
 						 1.0, 0);
 	gtk_box_pack_start (GTK_BOX(whbox1), copies->last_spin, FALSE, FALSE, 0);
@@ -251,11 +248,10 @@ first_spin_cb (GtkSpinButton * spinbutton,
 	    gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON
 					      (copies->last_spin));
 
-	gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON (copies->last_spin))->
-	    lower = first;
+	gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON (copies->last_spin))->lower = first;
 
 	gl_wdgt_mini_preview_highlight_range (GL_WDGT_MINI_PREVIEW(copies->mini_preview),
-					 first, last);
+                                              first, last);
 }
 
 /*--------------------------------------------------------------------------*/
@@ -275,11 +271,10 @@ last_spin_cb (GtkSpinButton * spinbutton,
 	    gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON
 					      (copies->last_spin));
 
-	gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON (copies->first_spin))->
-	    upper = last;
+	gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON (copies->first_spin))->upper = last;
 
 	gl_wdgt_mini_preview_highlight_range (GL_WDGT_MINI_PREVIEW(copies->mini_preview),
-					 first, last);
+                                              first, last);
 }
 
 /*--------------------------------------------------------------------------*/
